//=============================================================================
// Custom Game Over, version 1.0.2
// by McKathlin
// Kath_GameOver.js
// Last Update: 2016.01.18
//=============================================================================

/*:
 * @plugindesc Change what happens when the party dies or Game Over is called.
 * 
 * @param Show Game Over Scene
 * @desc Whether to show Scene_Gameover. If false, only a brief fade to black
 * is seen.
 * @default true
 * 
 * @param Game Over Common Event ID
 * @desc The ID of the common event to run after (or instead of)
 * the Game Over scene. Leave blank for default game over behavior.
 * @default 
 * 
 * @help This plugin is designed to play well by itself and with other plugins.
 * 
 * =========================
 * = Show Game Over Scene  =
 * =========================
 * RPG Maker's default behavior takes the player to Scene_GameOver on
 * party death or on a scene processing call to Scene_GameOver.
 * This processing shows a Game Over screen.
 * After the player sees the screen and presses any key,
 * the game exits to the title screen.
 * 
 * If Show Game Over Scene is set to false, control will technically flow
 * to Scene_GameOver, but the player will only notice a brief fade to black
 * before the next scene loads.
 * 
 * Whether or not the Game Over screen shows, which scene is next depends on
 * whether the Game Over Common Event is set, and what it is set to.
 * 
 * ==============================
 * = Game Over Common Event ID  =
 * ==============================
 * Assigning a Game Over common event makes gameplay continue after
 * the party loses, instead of RPG Maker's default behavior of returning the
 * party to the title screen. Open the database to Common Events to find the
 * ID of the common event to call on game over, and enter this ID number as
 * the parameter.
 * 
 * In the content of the common event, the game designer can customize what
 * happens when the party dies or reaches an event-dictated Game Over state.
 * The Game Over common event might do some of the following things:
 * * Take away gold and/or items
 * * Return the player to a safe place
 * * Restore HP to one or more party members
 * * Have the party's rescuer say something
 * * ...anything that suits this game!
 *
 * IMPORTANT: When control flows to the Game Over common event,
 * the screen will start blacked out. This gives the event time to handle
 * transfers and other processing before showing the player the screen.
 * Once those things are ready, remember to fade in!
 */

var Imported = Imported || {};
Imported.Kath_GameOver = true;

var Kath = Kath || {};
Kath.GameOver = Kath.GameOver || {};

Kath.Parameters = PluginManager.parameters('Kath_GameOver');
Kath.Param = Kath.Param || {};

Kath.Param.ShowGameOverScene =
	Boolean(eval(String(Kath.Parameters['Show Game Over Scene']).toLowerCase()));
Kath.Param.GameOverCommonEventID =
	Number.parseInt(Kath.Parameters['Game Over Common Event ID']);

//--------------------------------------------------------------------
// Scene_GameOver create - extended method
// Moves straight to next scene if Show Game Over Scene is false.
// Otherwise, behaves normally.
Kath.GameOver.Scene_Gameover_create = Scene_Gameover.prototype.create;
Scene_Gameover.prototype.create = function() {
    if (Kath.Param.ShowGameOverScene) {
    	Kath.GameOver.Scene_Gameover_create.call(this);
    } else {
    	Scene_Base.prototype.create.call(this);
    	// In case of plugins that use _backSprite....
    	this._backSprite = new Sprite();
    	this._backSprite.bitmap = ImageManager.loadEmptyBitmap();
    	this.gotoTitle();
    }
};

//--------------------------------------------------------------------
// Scene_GameOver start - extended method
// Skips all processing if Show Game Over Scene is false.
// Otherwise, behaves normally.
Kath.GameOver.Scene_Gameover_start = Scene_Gameover.prototype.start;
Scene_Gameover.prototype.start = function() {
	if (Kath.Param.ShowGameOverScene) {
		Kath.GameOver.Scene_Gameover_start.call(this);
	}
	// If nothing to show, then nothing to do here.
};

//--------------------------------------------------------------------
// Scene_GameOver is triggered - extended method
// No key press required if Show Game Over Scene is false.
Kath.GameOver.Scene_Gameover_isTriggered = Scene_Gameover.prototype.isTriggered;
Scene_Gameover.prototype.isTriggered = function() {
    return !Kath.Param.ShowGameOverScene ||
    	Kath.GameOver.Scene_Gameover_isTriggered.call(this);
};

//----------------------------------------------------------------------------
// Scene_GameOver goto title - extended method
// If a Game Over Common Event is defined, call it instead of going to title.
Kath.GameOver.Scene_Gameover_gotoTitle = Scene_Gameover.prototype.gotoTitle;
Scene_Gameover.prototype.gotoTitle = function() {
	if (Kath.Param.GameOverCommonEventID > 0) {
		$gameScreen.startFadeOut(1); // instant
		$gameParty.reviveBattleMembers();
		$gameTemp.reserveCommonEvent(Kath.Param.GameOverCommonEventID);
		SceneManager.goto(Scene_Map);
	}
	else {
		Kath.GameOver.Scene_Gameover_gotoTitle.call(this);
	}
};